@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -26,44 +26,48 @@ from time import time
 from getpass import getuser
 from signal import signal, SIGINT, SIGTERM
 
+from eolie.application_night import NightApplication
 from eolie.settings import Settings
 from eolie.window import Window
 from eolie.art import Art
+from eolie.content_blocker_ad import AdContentBlocker
+from eolie.content_blocker_popups import PopupsContentBlocker
+from eolie.content_blocker_images import ImagesContentBlocker
+from eolie.content_blocker_medias import MediasContentBlocker
+from eolie.content_blocker_scripts import ScriptsContentBlocker
+from eolie.content_blocker_phishing import PhishingContentBlocker
 from eolie.database_history import DatabaseHistory
 from eolie.database_bookmarks import DatabaseBookmarks
-from eolie.database_adblock import DatabaseAdblock
-from eolie.database_exceptions import DatabaseExceptions
 from eolie.database_settings import DatabaseSettings
-from eolie.database_phishing import DatabasePhishing
 from eolie.sqlcursor import SqlCursor
 from eolie.context import Context
 from eolie.search import Search
 from eolie.download_manager import DownloadManager
 from eolie.menu_pages import PagesMenu
-from eolie.helper_dbus import DBusHelper
 from eolie.helper_task import TaskHelper
 from eolie.define import EOLIE_DATA_PATH, TimeSpan, TimeSpanValues, LoadingType
 from eolie.utils import is_unity, wanted_loading_type
 from eolie.logger import Logger
+from eolie.webview_state import WebViewState
 
 
-class Application(Gtk.Application):
+class Application(Gtk.Application, NightApplication):
     """
         Eolie application
     """
 
     __FAVICONS_PATH = "/tmp/eolie_%s" % getuser()
 
-    def __init__(self, version, extension_dir):
+    def __init__(self, version, data_dir):
         """
             Create application
             @param version as str
-            @param extension_dir as str
+            @param data_dir as str
         """
         self.__version = version
         self.__state_cache = []
-        self.__pinned = []
-        self.__ephemeral_context = None
+        self.__data_dir = data_dir
+        self.__content_blockers = []
         signal(SIGINT, lambda a, b: self.quit())
         signal(SIGTERM, lambda a, b: self.quit())
         # Set main thread name
@@ -96,7 +100,6 @@ class Application(Gtk.Application):
                     GLib.setenv("SSL_CERT_FILE", path, True)
                     break
         self.sync_worker = None  # Not initialised
-        self.__extension_dir = extension_dir
         self.show_tls = False
         self.cursors = {}
         GLib.set_application_name('Eolie')
@@ -121,18 +124,6 @@ class Application(Gtk.Application):
         if self.get_is_remote():
             Gdk.notify_startup_complete()
 
-    def update_default_style_sheet(self):
-        """
-            Should be called on startup
-        """
-        rules = self.adblock.get_default_css_rules()
-        self.__default_style_sheet = WebKit2.UserStyleSheet(
-            rules,
-            WebKit2.UserContentInjectedFrames.ALL_FRAMES,
-            WebKit2.UserStyleLevel.USER,
-            None,
-            None)
-
     def do_startup(self):
         """
             Init application
@@ -141,17 +132,19 @@ class Application(Gtk.Application):
         if not self.get_windows():
             self.__init()
 
-    def get_new_window(self, size=None, maximized=False):
+    def get_new_window(self):
         """
             Return a new window
             @return Window
         """
+        size = (800, 600)
+        is_maximized = False
         windows = self.get_windows()
-        if windows and size is None:
+        if windows:
             active_window = self.active_window
             size = active_window.size
-            maximized = active_window.is_maximized()
-        window = Window(self, size, maximized)
+            is_maximized = active_window.is_maximized()
+        window = Window(size, is_maximized)
         window.connect('delete-event', self.__on_delete_event)
         window.show()
         return window
@@ -163,8 +156,8 @@ class Application(Gtk.Application):
             @param value as GLib.Variant
         """
         for window in self.get_windows():
-            for view in window.container.views:
-                view.webview.set_setting(key, value)
+            for webview in window.container.webviews:
+                webview.set_setting(key, value)
 
     def update_unity_badge(self, fraction=None):
         """
@@ -182,22 +175,6 @@ class Application(Gtk.Application):
                 self.__unity.set_property("progress", fraction)
                 self.__unity.set_property("progress_visible", fraction != 1.0)
 
-    def add_to_pinned(self, uri):
-        """
-            Add uri to pinned pages
-            @param uri as str
-        """
-        if uri not in self.__pinned:
-            self.__pinned.append(uri)
-
-    def remove_from_pinned(self, uri):
-        """
-            Remove uri from pinned pages
-            @param uri as str
-        """
-        if uri in self.__pinned:
-            self.__pinned.remove(uri)
-
     def quit(self, vacuum=False):
         """
             Quit application
@@ -208,7 +185,8 @@ class Application(Gtk.Application):
             window.hide()
         # Stop pending tasks
         self.download_manager.cancel()
-        self.adblock.stop()
+        for content_blocker in self.__content_blockers:
+            content_blocker.stop()
         # Clear history
         active_id = str(self.settings.get_enum("history-storage"))
         if active_id != TimeSpan.FOREVER:
@@ -218,8 +196,7 @@ class Application(Gtk.Application):
             self.history.clear_to(int(atime))
 
         if self.sync_worker is not None:
-            if self.sync_worker.syncing:
-                self.sync_worker.stop()
+            self.sync_worker.stop()
             self.sync_worker.save_pendings()
         if vacuum:
             task_helper = TaskHelper()
@@ -228,13 +205,27 @@ class Application(Gtk.Application):
         else:
             Gio.Application.quit(self)
 
+    def get_content_blocker(self, name):
+        """
+            Get content blocker by name
+            @param name as str
+            @return ContentBlocker
+        """
+        for content_blocker in self.__content_blockers:
+            if content_blocker.name == name:
+                return content_blocker
+
     @property
-    def pinned(self):
+    def content_filters(self):
         """
-            Get pinned pages
-            @return [str]
+            Get content filters
+            @return [WebKit2.UserContentFilter]
         """
-        return self.__pinned
+        filters = []
+        for content_blocker in self.__content_blockers:
+            if content_blocker.enabled:
+                filters.append(content_blocker.filter)
+        return filters
 
     @property
     def start_page(self):
@@ -244,31 +235,14 @@ class Application(Gtk.Application):
         """
         value = self.settings.get_value("start-page").get_string()
         if value in ["popular_hist", "popular_book"]:
-            value = "populars://"
+            start_page = "populars://"
         elif value == "blank":
-            value = "about:blank"
+            start_page = "about:blank"
+        elif value == "search":
+            start_page = self.search.uri
         else:
-            value = self.search.uri
-        return value
-
-    @property
-    def ephemeral_context(self):
-        """
-            Return ephemeral context
-            @return WebKit2.WebContext
-        """
-        if self.__ephemeral_context is None:
-            self.__ephemeral_context = WebKit2.WebContext.new_ephemeral()
-            Context(self.__ephemeral_context)
-        return self.__ephemeral_context
-
-    @property
-    def default_style_sheet(self):
-        """
-            Get default style sheet
-            @return WebKit2.UserStyleSheet
-        """
-        return self.__default_style_sheet
+            start_page = value
+        return start_page
 
     @property
     def active_window(self):
@@ -287,17 +261,18 @@ class Application(Gtk.Application):
         return self.get_windows()
 
     @property
-    def extension_dir(self):
+    def data_dir(self):
         """
-            Extension dir path
+            Get data dir
             @return str
         """
-        return self.__extension_dir
+        return self.__data_dir
 
     @property
     def favicons_path(self):
         """
             Cookies sqlite DB path
+            @return str
         """
         return self.__FAVICONS_PATH
 
@@ -309,21 +284,13 @@ class Application(Gtk.Application):
             Init main application
         """
         self.settings = Settings.new()
+        NightApplication.__init__(self)
 
-        # Init extensions
-        current_path = GLib.getenv("PYTHONPATH")
-        new_path = self.__extension_dir
-        if current_path:
-            new_path = new_path + ':' + current_path
-        GLib.setenv("PYTHONPATH", new_path, True)
-
-        # Add a global DBus helper
-        self.helper = DBusHelper()
         # First init sync worker
         from eolie.firefox_sync import SyncWorker
         if SyncWorker.check_modules():
             self.sync_worker = SyncWorker()
-            self.sync_worker.sync_loop()
+            self.sync_worker.pull_loop()
         else:
             self.sync_worker = None
         cssProviderFile = Gio.File.new_for_uri(
@@ -337,21 +304,18 @@ class Application(Gtk.Application):
         self.history = DatabaseHistory()
         self.bookmarks = DatabaseBookmarks()
         self.websettings = DatabaseSettings()
-        self.adblock = DatabaseAdblock()
-        self.adblock.create_db()
-        self.adblock.update()
-        self.phishing = DatabasePhishing()
-        self.phishing.create_db()
-        self.adblock_exceptions = DatabaseExceptions("adblock")
-        # Do not remove this!
-        self.update_default_style_sheet()
-        self.popup_exceptions = DatabaseExceptions("popups")
-        self.image_exceptions = DatabaseExceptions("images")
-        if self.settings.get_user_value("jsblock") is not None:
-            self.js_exceptions = DatabaseExceptions("js")
-        else:
-            self.js_exceptions = None
-        self.phishing.update()
+        for cls in [AdContentBlocker,
+                    PopupsContentBlocker,
+                    ImagesContentBlocker,
+                    MediasContentBlocker,
+                    ScriptsContentBlocker,
+                    PhishingContentBlocker]:
+            content_blocker = cls()
+            content_blocker.connect("set-filter",
+                                    self.__on_content_blocker_set_filter)
+            content_blocker.connect("unset-filter",
+                                    self.__on_content_blocker_unset_filter)
+            self.__content_blockers.append(content_blocker)
         self.art = Art()
 
         # Get a default user agent for search
@@ -385,8 +349,6 @@ class Application(Gtk.Application):
         shortcut_action.connect('activate', self.__on_shortcut_action)
         self.add_action(shortcut_action)
         self.set_accels_for_action("win.shortcut::expose", ["<Alt>e"])
-        self.set_accels_for_action("win.exceptions::site", ["<Control>e"])
-        self.set_accels_for_action("win.shortcut::jsblock", ["<Control>j"])
         self.set_accels_for_action("win.shortcut::show_sidebar", ["F9"])
         self.set_accels_for_action("win.shortcut::uri",
                                    ["<Control>l", "<Control>b"])
@@ -411,8 +373,8 @@ class Application(Gtk.Application):
         self.set_accels_for_action("win.shortcut::history", ["<Control>h"])
         self.set_accels_for_action("win.shortcut::search", ["<Control>k"])
         self.set_accels_for_action("win.shortcut::fullscreen", ["F11"])
-        self.set_accels_for_action("app.settings",
-                                   ["<Control>s"])
+        self.set_accels_for_action("win.shortcut::settings",
+                                   ["<Control>comma"])
         self.set_accels_for_action("win.shortcut::backward",
                                    ["<Alt>Left", "Back"])
         self.set_accels_for_action("win.shortcut::forward",
@@ -452,72 +414,23 @@ class Application(Gtk.Application):
                 sql.isolation_level = None
                 sql.execute("VACUUM")
                 sql.isolation_level = ""
-            with SqlCursor(self.adblock) as sql:
-                sql.isolation_level = None
-                sql.execute("VACUUM")
-                sql.isolation_level = ""
-            with SqlCursor(self.phishing) as sql:
-                sql.isolation_level = None
-                sql.execute("VACUUM")
-                sql.isolation_level = ""
         except Exception as e:
             Logger.error("Application::__vacuum(): %s ", e)
         self.art.vacuum()
 
-    def __get_state(self, window):
-        """
-            Return state for window
-            @param window as Window
-            @return {}
-        """
-        def get_state_for_webview(webview):
-            uri = webview.uri
-            parsed = urlparse(uri)
-            if parsed.scheme in ["http", "https"]:
-                ephemeral = webview.ephemeral
-                state = webview.get_session_state().serialize()
-                return (uri, webview.title, webview.atime,
-                        ephemeral, state.get_data())
-            else:
-                return None
-
-        window_state = {}
-        window_state["id"] = str(window)
-        window_state["size"] = window.size
-        window_state["maximized"] = window.is_maximized()
-        session_states = []
-        if self.settings.get_value("remember-session"):
-            # Save current first, will be loaded first on restore
-            current = window.container.current.webview
-            state = get_state_for_webview(current)
-            if state is not None:
-                session_states.append(state)
-            # Do not get view from container to save order
-            for view in window.container.views:
-                if view.webview == current or view.destroying:
-                    continue
-                state = get_state_for_webview(view.webview)
-                if state is not None:
-                    session_states.append(state)
-        window_state["states"] = session_states
-        window_state["sites"] = window.container.sites_manager.sort
-        return window_state
-
     def __save_state(self):
         """
             Save windows state
         """
         try:
             window_states = []
-            for window in self.get_windows():
-                window_state = self.__get_state(window)
-                window_states.append(window_state)
+            for window in self.windows:
+                state = window.state
+                window_states.append(state)
             for window_state in self.__state_cache:
                 window_states.append(window_state)
             dump(window_states,
                  open(EOLIE_DATA_PATH + "/session_states.bin", "wb"))
-            dump(self.__pinned,
-                 open(EOLIE_DATA_PATH + "/pinned.bin", "wb"))
         except Exception as e:
             Logger.error("Application::__save_state(): %s", e)
 
@@ -527,74 +440,68 @@ class Application(Gtk.Application):
             @param window_id as str
         """
         for state in self.__state_cache:
-            if state["id"] == window_id:
+            if state.wid == window_id:
                 self.__state_cache.remove(state)
                 break
 
-    def __create_initial_windows(self, foreground):
+    def __restore_state(self):
         """
-            Create initial windows based on saved session
-            @param foreground  as bool if foreground loading allowed
+            Restore state
+            @return active window as Window
         """
-        size = (800, 600)
-        maximized = False
-        pinned = []
         try:
-            pinned = load(
-                open(EOLIE_DATA_PATH + "/pinned.bin", "rb"))
+            pinned_netlocs = self.websettings.get_pinned_netlocs()
+            startup_uris = self.bookmarks.get_startup_uris()
+            # Add saved webviews
+            from eolie.window_state import WindowState, WindowStateStruct
             window_states = load(
                 open(EOLIE_DATA_PATH + "/session_states.bin", "rb"))
-            if self.settings.get_value("remember-session"):
-                for window_state in window_states:
-                    window = self.get_new_window(window_state["size"],
-                                                 window_state["maximized"])
-                    window.container.sites_manager.set_initial_sort(
-                        window_state["sites"])
-                    items = []
-                    i = 0 if foreground else 1
-                    for (uri, title, atime,
-                         ephemeral, state) in window_state["states"]:
-                        if uri in pinned:
-                            pinned.remove(uri)
-                        loading_type = wanted_loading_type(i)
-                        webkit_state = WebKit2.WebViewSessionState(
-                            GLib.Bytes.new(state))
-                        items.append((uri, title, atime, ephemeral,
-                                      webkit_state, loading_type))
-                        i += 1
-                    if window_state["states"]:
-                        window.container.add_webviews(items)
-                    else:
-                        window.container.add_webview(
-                            self.start_page,
-                            LoadingType.FOREGROUND,
-                            False)
-            elif window_states:
-                size = window_states[0]["size"]
-                maximized = window_states[0]["maximized"]
+            state = WindowStateStruct()
+            for state in window_states:
+                # If webview to restore
+                if state.webview_states:
+                    window = WindowState.new_from_state(state)
+                    for webview_state in state.webview_states:
+                        parsed = urlparse(webview_state.uri)
+                        if parsed.netloc in pinned_netlocs:
+                            pinned_netlocs.remove(parsed.netloc)
+                        if webview_state.uri in startup_uris:
+                            startup_uris.remove(webview_state.uri)
+                        webview = WebViewState.new_from_state(webview_state,
+                                                              window)
+                        webview.show()
+                        webview.set_shown(True)
+                        loading_type = wanted_loading_type(
+                            len(window.container.webviews))
+                        window.container.add_webview(webview, loading_type)
+                        session = WebKit2.WebViewSessionState(
+                            GLib.Bytes.new(webview_state.session))
+                        webview.restore_session_state(session)
+                    window.connect("delete-event", self.__on_delete_event)
+                    window.show()
+            # Add a default window
+            if not self.windows:
+                window = WindowState.new_from_state(state)
+                window.connect("delete-event", self.__on_delete_event)
+                window.show()
+            # Add pinned websites
+            for netloc in pinned_netlocs:
+                webview = window.container.add_webview_for_uri(
+                                                     "http://%s" % netloc,
+                                                     LoadingType.BACKGROUND)
+                webview.set_shown(True)
+            # Add startup webpage
+            for uri in startup_uris:
+                webview = window.container.add_webview_for_uri(
+                                                     uri,
+                                                     LoadingType.BACKGROUND)
+                webview.set_shown(True)
+            # Make first webview visible
+            if window.container.webviews:
+                GLib.idle_add(window.container.set_visible_webview,
+                              window.container.webviews[0])
         except Exception as e:
-            Logger.error("Application::__create_initial_windows(): %s", e)
-        # Create a window if None available and setup pinned pages
-        # We already removed saved states from pinned
-        windows = self.get_windows()
-        if windows:
-            window = windows[0]
-        else:
-            window = self.get_new_window(size, maximized)
-            window.container.add_webview(
-                            self.start_page,
-                            LoadingType.FOREGROUND,
-                            False)
-        for uri in pinned:
-            window.container.add_webview(uri,
-                                         LoadingType.BACKGROUND,
-                                         False)
-        # Really setup pinned
-        try:
-            self.__pinned = load(
-                    open(EOLIE_DATA_PATH + "/pinned.bin", "rb"))
-        except:
-            self.__pinned = []
+            Logger.error("Application::__restore_state(): %s", e)
 
     def __on_handle_local_options(self, app, options):
         """
@@ -620,41 +527,45 @@ class Application(Gtk.Application):
             self.show_tls = True
         if options.contains("disable-artwork-cache"):
             self.art.disable_cache()
-        ephemeral = options.contains("private")
-        if not self.get_windows():
-            self.__create_initial_windows(len(args) < 2)
-            active_window = self.active_window
-        elif options.contains("new") or len(args) == 1:
+
+        # FIXME
+        # is_ephemeral = options.contains("private")
+
+        # Only restore state on first run
+        if not self.windows:
+            self.__restore_state()
+
+        # Setup at least one window
+        if not self.windows or options.contains("new"):
             active_window = self.get_new_window()
         else:
-            active_window = self.active_window
+            active_window = self.windows[0]
+
         # Open command line args
-        if len(args) > 1:
-            items = []
-            i = 0
-            for uri in args[1:]:
-                # Transform path to uri
-                parsed = urlparse(uri)
-                if not parsed.scheme:
-                    if uri.startswith('/'):
-                        uri = "file://%s" % uri
-                    else:
-                        uri = "http://%s" % uri
-                loading_type = wanted_loading_type(i)
-                items.append((uri, uri, None, ephemeral, None, loading_type))
-                i += 1
-            active_window.container.add_webviews(items)
-            active_window.present()
+        for uri in args[1:]:
+            # Transform path to uri
+            parsed = urlparse(uri)
+            if not parsed.scheme:
+                if uri.startswith('/'):
+                    uri = "file://%s" % uri
+                else:
+                    uri = "http://%s" % uri
+            loading_type = wanted_loading_type(
+                len(active_window.container.webviews))
+            active_window.container.add_webview_for_uri(uri, loading_type)
+
         # Add default start page
-        if not active_window.container.views:
-            active_window.container.add_webview(self.start_page,
-                                                LoadingType.FOREGROUND,
-                                                ephemeral)
+        if not active_window.container.webviews:
+            loading_type = wanted_loading_type(
+                len(active_window.container.webviews))
+            active_window.container.add_webview_for_uri(self.start_page,
+                                                        loading_type)
         if self.settings.get_value("debug"):
             WebKit2.WebContext.get_default().get_plugins(None,
                                                          self.__on_get_plugins,
                                                          None)
         Gdk.notify_startup_complete()
+        active_window.present()
         return 0
 
     def __close_window(self, window):
@@ -662,37 +573,40 @@ class Application(Gtk.Application):
             Close window
         """
         if len(self.get_windows()) > 1:
-            state = self.__get_state(window)
-            self.__state_cache.append(state)
-            GLib.timeout_add(10000, self.__clean_state_cache, state["id"])
+            state = window.state
+            if state is not None:
+                self.__state_cache.append(window.state)
+                GLib.timeout_add(10000, self.__clean_state_cache, state.wid)
             window.destroy()
         else:
             self.quit(True)
 
-    def __try_closing(self, window, views):
+    def __try_closing(self, window, webviews):
         """
-            Try closing all views
+            Try closing all webviews
+            @param window as Window
+            @param webviews as [WebView]
         """
-        if views:
-            view = views.pop(0)
-            page_id = view.webview.get_page_id()
-            self.helper.call("FormsFilled", page_id, None,
-                             self.__on_forms_filled, window, views)
+        if webviews:
+            webview = webviews.pop(0)
+            webview.run_javascript("document.activeElement.tagName;", None,
+                                   self.__on_get_active_element,
+                                   webviews, window)
         else:
             self.__close_window(window)
 
-    def __on_forms_filled(self, source, result, window, views):
+    def __on_get_active_element(self, source, result, webviews, window):
         """
             Ask user to close view, if ok, close view
             @param source as GObject.Object
             @param result as Gio.AsyncResult
+            @param webviews as [WebView]
             @param window as Window
-            @param views as [View]
         """
-        def on_response_id(dialog, response_id, window, views, self):
+        def on_response_id(dialog, response_id, window, webviews, self):
             if response_id == Gtk.ResponseType.CLOSE:
-                if views:
-                    self.__try_closing(window, views)
+                if webviews:
+                    self.__try_closing(window, webviews)
                 else:
                     self.__close_window(window)
             dialog.destroy()
@@ -704,8 +618,9 @@ class Application(Gtk.Application):
             dialog.response(Gtk.ResponseType.CANCEL)
 
         try:
-            result = source.call_finish(result)[0]
-            if result:
+            data = source.run_javascript_finish(result)
+            name = data.get_js_value().to_string()
+            if name == "TEXTAREA":
                 builder = Gtk.Builder()
                 builder.add_from_resource("/org/gnome/Eolie/QuitDialog.ui")
                 dialog = builder.get_object("dialog")
@@ -714,13 +629,14 @@ class Application(Gtk.Application):
                 cancel = builder.get_object("cancel")
                 label.set_text(_("Do you really want to quit Eolie?"))
                 dialog.set_transient_for(window)
-                dialog.connect("response", on_response_id, window, views, self)
+                dialog.connect("response", on_response_id,
+                               window, webviews, self)
                 close.connect("clicked", on_close, dialog)
                 cancel.connect("clicked", on_cancel, dialog)
                 dialog.run()
             else:
-                if views:
-                    self.__try_closing(window, views)
+                if webviews:
+                    self.__try_closing(window, webviews)
                 else:
                     self.__close_window(window)
         except Exception as e:
@@ -749,7 +665,7 @@ class Application(Gtk.Application):
         """
         def on_response_id(dialog, response_id, window):
             if response_id == Gtk.ResponseType.CLOSE:
-                self.__try_closing(window, window.container.views)
+                self.__try_closing(window, window.container.webviews)
             dialog.destroy()
 
         def on_close(widget, dialog):
@@ -773,7 +689,7 @@ class Application(Gtk.Application):
             cancel.connect("clicked", on_cancel, dialog)
             dialog.run()
         else:
-            self.__try_closing(window, window.container.views)
+            self.__try_closing(window, window.container.webviews)
         return True
 
     def __on_activate(self, application):
@@ -793,5 +709,28 @@ class Application(Gtk.Application):
         string = param.get_string()
         if string == "new_window":
             window = self.get_new_window()
-            window.container.add_webview(self.start_page,
-                                         LoadingType.FOREGROUND)
+            window.container.add_webview_for_uri(
+                self.start_page, LoadingType.FOREGROUND)
+
+    def __on_content_blocker_set_filter(self, content_blocker, content_filter):
+        """
+            Add filter to content manager
+            @param content_blocker as ContentBlocker
+            @param content_filter as WebKit2.UserContentFilter
+        """
+        for window in self.windows:
+            for webview in window.container.webviews:
+                content_manager = webview.get_user_content_manager()
+                content_manager.add_filter(content_filter)
+
+    def __on_content_blocker_unset_filter(self, content_blocker,
+                                          content_filter):
+        """
+            Remove filter from content manager
+            @param content_blocker as ContentBlocker
+            @param content_filter as WebKit2.UserContentFilter
+        """
+        for window in self.windows:
+            for webview in window.container.webviews:
+                content_manager = webview.get_user_content_manager()
+                content_manager.remove_filter(content_filter)
